"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.QuestionRecord = void 0;
const errors_1 = require("../utils/errors");
const db_1 = require("../utils/db");
const uuid_1 = require("uuid");
class QuestionRecord {
    constructor(obj) {
        if (!obj.name || obj.name.length > 100) {
            throw new errors_1.ValidationError('The question cannot be empty or longer than 100 characters!');
        }
        if (!(obj.type === 'open' || obj.type === 'radio' || obj.type === 'checkbox')) {
            throw new errors_1.ValidationError('Question type must be one of the given options!');
        }
        this.id = obj.id;
        this.name = obj.name;
        this.type = obj.type;
        this.answers = obj.answers;
    }
    static getOne(id) {
        return __awaiter(this, void 0, void 0, function* () {
            const [results] = yield db_1.pool.execute("SELECT * FROM `questions` WHERE `id` = :id", {
                id,
            });
            return results.length === 0 ? null : new QuestionRecord(results[0]);
        });
    }
    static getAll() {
        return __awaiter(this, void 0, void 0, function* () {
            const [results] = yield db_1.pool.execute("SELECT * FROM `questions`");
            return results.map(result => {
                const { id, name, type } = result;
                return { id, name, type };
            });
        });
    }
    insert() {
        return __awaiter(this, void 0, void 0, function* () {
            const ans = this.answers ? JSON.stringify(this.answers.map(a => ({ id: (0, uuid_1.v4)(), text: a.text, votes: 0 }))) : null;
            if (!this.id) {
                this.id = (0, uuid_1.v4)();
            }
            else {
                throw new Error('Cannot insert something that is already inserted!');
            }
            yield db_1.pool.execute("INSERT INTO `questions`(`id`, `name`, `type`, `answers`) VALUES(:id, :name, :type, :answers )", {
                id: this.id,
                name: this.name,
                type: this.type,
                answers: ans,
            });
        });
    }
    update(answerBody) {
        return __awaiter(this, void 0, void 0, function* () {
            let ans;
            const answers = typeof this.answers === 'string' ? JSON.parse(this.answers) : this.answers;
            if (this.type === "open") {
                if (this.answers === null) {
                    ans = [{ "id": (0, uuid_1.v4)(), "text": answerBody[0] }];
                }
                else {
                    ans = [...answers, { "id": (0, uuid_1.v4)(), "text": answerBody[0] }];
                }
            }
            else {
                ans = answers.map((a) => (answerBody.indexOf(a.id) > -1 ? Object.assign(Object.assign({}, a), { votes: a.votes + 1 }) : a));
            }
            console.log(ans);
            yield db_1.pool.execute("UPDATE `questions` SET `answers` = :answers WHERE `id` = :id", {
                id: this.id,
                answers: JSON.stringify(ans),
            });
        });
    }
}
exports.QuestionRecord = QuestionRecord;
//# sourceMappingURL=question.record.js.map